@@ -1,4 +1,6 @@
 import json
+import cfnlint.core
+from cfnlint import Runner
 import itertools
 import os.path
 import hashlib
@@ -34,6 +36,21 @@ BASE_PATH = os.path.dirname(__file__)
 INPUT_FOLDER = os.path.join(BASE_PATH, 'input')
 OUTPUT_FOLDER = os.path.join(BASE_PATH, 'output')
 
+LINT_IGNORE_WARNINGS = [
+    'W2001', # unused parameters. Sometimes, SAM uses parameters and removes the param reference from the output template, but the parameter stays in the parameters section.
+    'W1001', # Ref/GetAtt with conditions. This incorrectly flags resources since it can't map conditions fully.
+    'E3001', # Check for resource availability in a region.
+    'W7001', # Check if mappings are used. Serverless::Application uses mappings, the output CFN doesn't use them anymore.
+    'W1020', # Sub isn't needed if it doesn't have a variable defined. SAM leaves `!Sub` in even if it tries to resolve variables.
+    'E2531', # we don't care if a runtime has been deprecated in our tests.
+    'E3038', # Serverless resources- test for invalid resources.
+]
+
+LINT_IGNORE_TESTS = [
+    'function_with_resource_refs', # Tests functionality of the translator in ways that result in improper GetAtt calls on CFN resources.
+    'api_with_canary_setting', # Has stage variable overrides for nonexistent stage variables.
+]
+
 
 def deep_sort_lists(value):
     """
@@ -119,12 +136,12 @@ def mock_sar_service_call(self, service_call_function, logical_id, *args):
     elif application_id == "expired":
         status = "EXPIRED"
     message = {
-        'ApplicationId': args[0], 
-        'CreationTime': 'x', 
-        'ExpirationTime': 'x', 
-        'SemanticVersion': '1.1.1', 
-        'Status': status, 
-        'TemplateId': 'id-xx-xx', 
+        'ApplicationId': args[0],
+        'CreationTime': 'x',
+        'ExpirationTime': 'x',
+        'SemanticVersion': '1.1.1',
+        'Status': status,
+        'TemplateId': 'id-xx-xx',
         'TemplateUrl': 'https://awsserverlessrepo-changesets-xxx.s3.amazonaws.com/signed-url'
     }
     return message
@@ -142,6 +159,7 @@ class TestTranslatorEndToEnd(TestCase):
         'basic_function',
         'basic_application',
         'application_preparing_state',
+        'application_with_intrinsics',
         'basic_layer',
         'cloudwatchevent',
         'cloudwatch_logs_with_ref',
@@ -156,21 +174,33 @@ class TestTranslatorEndToEnd(TestCase):
         'api_with_auth_all_maximum',
         'api_with_auth_all_minimum',
         'api_with_auth_no_default',
+        'api_with_default_aws_iam_auth',
+        'api_with_method_aws_iam_auth',
+        'api_with_aws_iam_auth_overrides',
         'api_with_method_settings',
         'api_with_binary_media_types',
+        'api_with_binary_media_types_definition_body',
         'api_with_minimum_compression_size',
         'api_with_resource_refs',
         'api_with_cors',
+        'api_with_cors_and_auth_no_preflight_auth',
+        'api_with_cors_and_auth_preflight_auth',
         'api_with_cors_and_only_methods',
         'api_with_cors_and_only_headers',
         'api_with_cors_and_only_origins',
         'api_with_cors_and_only_maxage',
         'api_with_cors_and_only_credentials_false',
         'api_with_cors_no_definitionbody',
+        'api_with_gateway_responses',
+        'api_with_gateway_responses_all',
+        'api_with_gateway_responses_minimal',
+        'api_with_gateway_responses_implicit',
+        'api_with_gateway_responses_string_status_code',
         'api_cache',
         'api_with_access_log_setting',
         'api_with_canary_setting',
         'api_with_xray_tracing',
+        'api_request_model',
         's3',
         's3_create_remove',
         's3_existing_lambda_notification_configuration',
@@ -197,6 +227,8 @@ class TestTranslatorEndToEnd(TestCase):
         'function_with_kmskeyarn',
         'function_with_alias',
         'function_with_alias_intrinsics',
+        'function_with_custom_codedeploy_deployment_preference',
+        'function_with_custom_conditional_codedeploy_deployment_preference',
         'function_with_disabled_deployment_preference',
         'function_with_deployment_preference',
         'function_with_deployment_preference_all_parameters',
@@ -211,6 +243,7 @@ class TestTranslatorEndToEnd(TestCase):
         'function_with_permissions_boundary',
         'function_with_policy_templates',
         'function_with_sns_event_source_all_parameters',
+        'global_handle_path_level_parameter',
         'globals_for_function',
         'globals_for_api',
         'globals_for_simpletable',
@@ -226,7 +259,7 @@ class TestTranslatorEndToEnd(TestCase):
         'implicit_api_with_many_conditions',
         'implicit_and_explicit_api_with_conditions',
         'api_with_cors_and_conditions_no_definitionbody',
-        'api_with_auth_and_conditions_all_max'
+        'api_with_auth_and_conditions_all_max',
       ],
       [
        ("aws", "ap-southeast-1"),
@@ -245,7 +278,8 @@ class TestTranslatorEndToEnd(TestCase):
         # To uncover unicode-related bugs, convert dict to JSON string and parse JSON back to dict
         manifest = json.loads(json.dumps(manifest))
         partition_folder = partition if partition != "aws" else ""
-        expected = json.load(open(os.path.join(OUTPUT_FOLDER,partition_folder, testcase + '.json'), 'r'))
+        expected_filepath = os.path.join(OUTPUT_FOLDER, partition_folder, testcase + '.json')
+        expected = json.load(open(expected_filepath, 'r'))
 
         with patch('boto3.session.Session.region_name', region):
             parameter_values = get_template_parameter_values()
@@ -262,12 +296,101 @@ class TestTranslatorEndToEnd(TestCase):
 
         print(json.dumps(output_fragment, indent=2))
 
+        # Run cfn-lint on translator test output files.
+        rules = cfnlint.core.get_rules([], LINT_IGNORE_WARNINGS, [])
+
         # Only update the deployment Logical Id hash in Py3.
         if sys.version_info.major >= 3:
             self._update_logical_id_hash(expected)
             self._update_logical_id_hash(output_fragment)
+            output_template = cfnlint.decode.cfn_json.load(expected_filepath)
+        else: # deprecation warning catching in py2
+            import warnings
+            with warnings.catch_warnings():
+                warnings.filterwarnings("ignore",category=DeprecationWarning)
+                output_template = cfnlint.decode.cfn_json.load(expected_filepath)
+        runner = cfnlint.Runner(rules, expected_filepath, output_template, [region])
+        matches = []
+
+        # Only run linter on normal/gov partitions. It errors on china regions
+        if testcase not in LINT_IGNORE_TESTS and partition != 'aws-cn':
+            matches = runner.run()
+        print('cfn-lint ({}): {}'.format(expected_filepath, matches))
 
         assert deep_sort_lists(output_fragment) == deep_sort_lists(expected)
+        assert len(matches) == 0
+
+    @parameterized.expand(
+      itertools.product([
+        'explicit_api_openapi_3',
+        'api_with_auth_all_maximum_openapi_3',
+        'api_with_cors_openapi_3',
+        'api_with_gateway_responses_all_openapi_3',
+        'api_with_open_api_version',
+        'api_with_open_api_version_2',
+        'api_with_auth_all_minimum_openapi',
+        'api_with_swagger_and_openapi_with_auth',
+        'api_with_openapi_definition_body_no_flag',
+        'api_request_model_openapi_3'
+      ],
+      [
+       ("aws", "ap-southeast-1"),
+       ("aws-cn", "cn-north-1"),
+       ("aws-us-gov", "us-gov-west-1")
+      ] # Run all the above tests against each of the list of partitions to test against
+      )
+    )
+    @patch('samtranslator.plugins.application.serverless_app_plugin.ServerlessAppPlugin._sar_service_call', mock_sar_service_call)
+    @patch('botocore.client.ClientEndpointBridge._check_default_region', mock_get_region)
+    def test_transform_success_openapi3(self, testcase, partition_with_region):
+        partition = partition_with_region[0]
+        region = partition_with_region[1]
+
+        manifest = yaml_parse(open(os.path.join(INPUT_FOLDER, testcase + '.yaml'), 'r'))
+        # To uncover unicode-related bugs, convert dict to JSON string and parse JSON back to dict
+        manifest = json.loads(json.dumps(manifest))
+        partition_folder = partition if partition != "aws" else ""
+        expected_filepath = os.path.join(OUTPUT_FOLDER, partition_folder, testcase + '.json')
+        expected = json.load(open(expected_filepath, 'r'))
+
+        with patch('boto3.session.Session.region_name', region):
+            parameter_values = get_template_parameter_values()
+            mock_policy_loader = MagicMock()
+            mock_policy_loader.load.return_value = {
+                'AWSLambdaBasicExecutionRole': 'arn:{}:iam::aws:policy/service-role/AWSLambdaBasicExecutionRole'.format(partition),
+                'AmazonDynamoDBFullAccess': 'arn:{}:iam::aws:policy/AmazonDynamoDBFullAccess'.format(partition),
+                'AmazonDynamoDBReadOnlyAccess': 'arn:{}:iam::aws:policy/AmazonDynamoDBReadOnlyAccess'.format(partition),
+                'AWSLambdaRole': 'arn:{}:iam::aws:policy/service-role/AWSLambdaRole'.format(partition),
+            }
+
+            output_fragment = transform(
+                manifest, parameter_values, mock_policy_loader)
+
+        print(json.dumps(output_fragment, indent=2))
+
+        # Run cfn-lint on translator test output files.
+        rules = cfnlint.core.get_rules([], LINT_IGNORE_WARNINGS, [])
+
+        # Only update the deployment Logical Id hash in Py3.
+        if sys.version_info.major >= 3:
+            self._update_logical_id_hash(expected)
+            self._update_logical_id_hash(output_fragment)
+            output_template = cfnlint.decode.cfn_json.load(expected_filepath)
+        else: # deprecation warning catching in py2
+            import warnings
+            with warnings.catch_warnings():
+                warnings.filterwarnings("ignore",category=DeprecationWarning)
+                output_template = cfnlint.decode.cfn_json.load(expected_filepath)
+        runner = cfnlint.Runner(rules, expected_filepath, output_template, [region])
+        matches = []
+
+        # Only run linter on normal/gov partitions. It errors on china regions
+        if testcase not in LINT_IGNORE_TESTS and partition != 'aws-cn':
+            matches = runner.run()
+        print('cfn-lint ({}): {}'.format(expected_filepath, matches))
+
+        assert deep_sort_lists(output_fragment) == deep_sort_lists(expected)
+        assert len(matches) == 0
 
     def _update_logical_id_hash(self, resources):
         """
@@ -330,8 +453,8 @@ class TestTranslatorEndToEnd(TestCase):
 
         # Update any Output References in the template
         for output_key, output_value in resources.get("Outputs", {}).items():
-            if output_value.get("Ref") in deployment_logical_id_dict:
-                output_value["Ref"] = deployment_logical_id_dict[output_value.get("Ref")]
+            if output_value.get("Value").get("Ref") in deployment_logical_id_dict:
+                output_value["Value"]["Ref"] = deployment_logical_id_dict[output_value.get("Value").get("Ref")]
 
     def _generate_new_deployment_hash(self, logical_id, dict_to_hash, rest_api_to_swagger_hash):
         data_bytes = json.dumps(dict_to_hash, separators=(',', ':'), sort_keys=True).encode("utf8")
@@ -340,19 +463,31 @@ class TestTranslatorEndToEnd(TestCase):
 
 @pytest.mark.parametrize('testcase', [
     'error_api_duplicate_methods_same_path',
+    'error_api_gateway_responses_nonnumeric_status_code',
+    'error_api_gateway_responses_unknown_responseparameter',
+    'error_api_gateway_responses_unknown_responseparameter_property',
     'error_api_invalid_auth',
+    'error_api_invalid_path',
     'error_api_invalid_definitionuri',
     'error_api_invalid_definitionbody',
+    'error_api_invalid_stagename',
+    'error_api_with_invalid_open_api_version',
     'error_api_invalid_restapiid',
+    'error_api_invalid_request_model',
     'error_application_properties',
     'error_application_does_not_exist',
     'error_application_no_access',
     'error_application_preparing_timeout',
     'error_cors_on_external_swagger',
     'error_invalid_cors_dict',
+    'error_invalid_findinmap',
+    'error_invalid_getatt',
     'error_cors_credentials_true_with_wildcard_origin',
     'error_cors_credentials_true_without_explicit_origin',
     'error_function_invalid_codeuri',
+    'error_function_invalid_api_event',
+    'error_function_invalid_autopublishalias',
+    'error_function_invalid_event_type',
     'error_function_invalid_layer',
     'error_function_no_codeuri',
     'error_function_no_handler',
@@ -373,6 +508,8 @@ class TestTranslatorEndToEnd(TestCase):
     'existing_permission_logical_id',
     'existing_role_logical_id',
     'error_invalid_template',
+    'error_resource_not_dict',
+    'error_resource_properties_not_dict',
     'error_globals_is_not_dict',
     'error_globals_unsupported_type',
     'error_globals_unsupported_property',
@@ -380,7 +517,9 @@ class TestTranslatorEndToEnd(TestCase):
     'error_function_policy_template_with_missing_parameter',
     'error_function_policy_template_invalid_value',
     'error_function_with_unknown_policy_template',
-    'error_function_with_invalid_policy_statement'
+    'error_function_with_invalid_policy_statement',
+    'error_function_with_invalid_condition_name',
+    'error_invalid_document_empty_semantic_version'
 ])
 @patch('boto3.session.Session.region_name', 'ap-southeast-1')
 @patch('samtranslator.plugins.application.serverless_app_plugin.ServerlessAppPlugin._sar_service_call', mock_sar_service_call)
@@ -615,119 +754,6 @@ class TestFunctionVersionWithParameterReferences(TestCase):
         return output_fragment
 
 
-class TestParameterValuesHandling(TestCase):
-    """
-    Test how user-supplied parameters & default template parameter values from template get merged
-    """
-
-    def test_add_default_parameter_values_must_merge(self):
-        parameter_values = {
-            "Param1": "value1"
-        }
-
-        sam_template = {
-            "Parameters": {
-                "Param2": {
-                    "Type": "String",
-                    "Default": "template default"
-                }
-            }
-        }
-
-        expected = {
-            "Param1": "value1",
-            "Param2": "template default"
-        }
-
-        sam_parser = Parser()
-        translator = Translator({}, sam_parser)
-        result = translator._add_default_parameter_values(sam_template,
-            parameter_values)
-        self.assertEqual(expected, result)
-
-    def test_add_default_parameter_values_must_override_user_specified_values(self):
-        parameter_values = {
-            "Param1": "value1"
-        }
-
-        sam_template = {
-            "Parameters": {
-                "Param1": {
-                    "Type": "String",
-                    "Default": "template default"
-                }
-            }
-        }
-
-        expected = {
-            "Param1": "value1"
-        }
-
-
-        sam_parser = Parser()
-        translator = Translator({}, sam_parser)
-        result = translator._add_default_parameter_values(sam_template, parameter_values)
-        self.assertEqual(expected, result)
-
-    def test_add_default_parameter_values_must_skip_params_without_defaults(self):
-        parameter_values = {
-            "Param1": "value1"
-        }
-
-        sam_template = {
-            "Parameters": {
-                "Param1": {
-                    "Type": "String"
-                },
-                "Param2": {
-                    "Type": "String"
-                }
-            }
-        }
-
-        expected = {
-            "Param1": "value1"
-        }
-
-        sam_parser = Parser()
-        translator = Translator({}, sam_parser)
-        result = translator._add_default_parameter_values(sam_template, parameter_values)
-        self.assertEqual(expected, result)
-
-
-    @parameterized.expand([
-        # Array
-        param(["1", "2"]),
-
-        # String
-        param("something"),
-
-        # Some other non-parameter looking dictionary
-        param({"Param1": {"Foo": "Bar"}}),
-
-        param(None)
-    ])
-    def test_add_default_parameter_values_must_ignore_invalid_template_parameters(self, template_parameters):
-        parameter_values = {
-            "Param1": "value1"
-        }
-
-        expected = {
-            "Param1": "value1"
-        }
-
-        sam_template = {
-            "Parameters": template_parameters
-        }
-
-
-        sam_parser = Parser()
-        translator = Translator({}, sam_parser)
-        result = translator._add_default_parameter_values(
-            sam_template, parameter_values)
-        self.assertEqual(expected, result)
-
-
 class TestTemplateValidation(TestCase):
 
     @patch('botocore.client.ClientEndpointBridge._check_default_region', mock_get_region)
@@ -764,6 +790,21 @@ class TestTemplateValidation(TestCase):
             translator = Translator({}, sam_parser)
             translator.translate(template, {})
 
+
+    @patch('botocore.client.ClientEndpointBridge._check_default_region', mock_get_region)
+    def test_throws_when_resources_not_all_dicts(self):
+        template = {
+            "Resources": {
+                "notadict": None,
+                "MyResource": {}
+            }
+        }
+
+        with self.assertRaises(InvalidDocumentException):
+            sam_parser = Parser()
+            translator = Translator({}, sam_parser)
+            translator.translate(template, {})
+
 class TestPluginsUsage(TestCase):
     # Tests if plugins are properly injected into the translator
 
@@ -851,7 +892,7 @@ class TestPluginsUsage(TestCase):
         resource_from_dict_mock.assert_called_with("MyTable",
                                                         manifest["Resources"]["MyTable"],
                                                         sam_plugins=sam_plugins_object_mock)
-        prepare_plugins_mock.assert_called_once_with(initial_plugins)
+        prepare_plugins_mock.assert_called_once_with(initial_plugins, {"AWS::Region": "ap-southeast-1"})
 
 def get_policy_mock():
     mock_policy_loader = MagicMock()
@@ -876,4 +917,3 @@ def get_resource_by_type(template, type):
 
 def get_exception_error_message(e):
     return reduce(lambda message, error: message + ' ' + error.message, e.value.causes, e.value.message)
-
